from django.conf import settings
from dateutil import tz

PROTOCOL = getattr(settings, 'ELASTIC_PROTOCOL', 'http')
HOST = getattr(settings, 'ELASTIC_HOST', 'localhost')
PORT = getattr(settings, 'ELASTIC_PORT', 9200)
USE_SSL = getattr(settings, 'ELASTIC_USE_SSL', False)
CURRENT_TIME_ZONE = tz.gettz(getattr(settings, 'TIME_ZONE', 'UTC'))
ALLOWED_STATUS = getattr(settings, 'ELASTIC_ALLOWED_STATUS', [])
ALLOWED_ALL = getattr(settings, 'ELASTIC_ALLOWED_ALL', False)
USER = getattr(settings, 'ELASTIC_USER', '')
SECRET = getattr(settings, 'ELASTIC_PASSWORD', '')



from tamarin.elasticsearch.client import ElasticSearchClient
from .config import CURRENT_TIME_ZONE, ALLOWED_STATUS
from tamarin.elasticsearch.utils.functions import validate_status, var2str
from tamarin.command import Command
from sentry_sdk import capture_exception
import uuid
import datetime
from tamarin.logs.models import Report
import traceback


class ElasticSearchCommand(Command):
    def __init__(self, response, status_code, index, doc, args=None, params=None, headers=None):
        super(ElasticSearchCommand, self).__init__()
        if headers is None:
            headers = {}
        if params is None:
            params = {}
        if args is None:
            args = {}
        self._response = response
        self._status_code = status_code
        self._index = index
        self._doc = doc
        self._args = args
        self._params = params
        self._headers = headers

    def execute(self):
        try:
            if validate_status(self._status_code):
                response = self._response
                metadata = response.get('metadata')
                transaction = metadata.pop('transaction', {})
                created_at = datetime.datetime.now(tz=CURRENT_TIME_ZONE)
                index = self._index
                doc = self._doc
                args = self._args
                params = self._params
                headers = self._headers
                client = ElasticSearchClient.get_client()
                elastic_id = metadata.get('id', F"{uuid.uuid4()}-{created_at.timestamp()}")
                body = {
                    'status': response.get('status'),
                    'data': self.clean_data_type(response.get('data')),
                    'metadata': metadata,
                    'transaction': self.clean_data_type(transaction),
                    'created_at': created_at,
                    'args': args
                }
                if response.get('error'):
                    error = response.get('error')
                    error = {
                        'messages': self.clean_data_type(error.get('message')),
                        'code': error.get('code')
                    }
                    body.update({'error': error})
                body = var2str(body)
                client.index(
                    index=index,
                    body=body,
                    doc_type=doc,
                    id=elastic_id,
                    params=params,
                    headers=headers
                )
                return {'elastic_id': elastic_id}
            else:
                return {'elastic_id': 'not allowed'}
        except Exception as e:
            capture_exception(e)
            report = Report.objects.create(
                index='elastic-exception',
                status=500,
                text=traceback.format_exc()
            )
            return {
                'elastic_id': 'exception occured',
                'report': report.id,
            }

    def clean_data_type(self, data):
        if data is None:
            return []
        if type(data) == list:
            return data
        else:
            return [data]



from threading import Lock, Thread
from django.http import HttpRequest, HttpResponse
from django.test import TestCase

from nose.tools import eq_

from tamarin.database.router.utils import (PRIMARY_DB_ALIAS, get_replica)
from tamarin.database.router.pinning_replica_router import PinningReplicaRouter
from tamarin.database.router.replica_router import ReplicaRouter
from tamarin.database.middleware.pinning_router_middleware import (
    PINNING_COOKIE,
    PINNING_SECONDS,
    PinningRouterMiddleware
)
from tamarin.database.router.utils import (
    this_thread_is_pinned,
    pin_this_thread,
    unpin_this_thread,
    use_primary_db,
    db_write
)


class UnpinningTestCase(TestCase):
    """Test case that unpins the thread on tearDown"""

    def tearDown(self):
        unpin_this_thread()


class ReplicaRouterTests(TestCase):

    def test_db_for_read(self):
        eq_(ReplicaRouter().db_for_read(None), get_replica())
        # TODO: Test the round-robin functionality.

    def test_db_for_write(self):
        eq_(ReplicaRouter().db_for_write(None), PRIMARY_DB_ALIAS)

    def test_allow_syncdb(self):
        router = ReplicaRouter()
        assert router.allow_syncdb(PRIMARY_DB_ALIAS, None)
        assert not router.allow_syncdb(get_replica(), None)

    def test_allow_migrate(self):
        router = ReplicaRouter()
        assert router.allow_migrate(PRIMARY_DB_ALIAS, 'dummy')
        assert not router.allow_migrate(get_replica(), 'dummy')


class SettingsTests(TestCase):
    """Tests for default settings."""

    def test_cookie_default(self):
        """Check that the cookie name has the right default."""
        eq_(PINNING_COOKIE, 'multidb_pin_writes')

    def test_pinning_seconds_default(self):
        """Make sure the cookie age has the right default."""
        eq_(PINNING_SECONDS, 15)


class PinningTests(UnpinningTestCase):
    """Tests for "pinning" functionality, above and beyond what's inherited
    from ReplicaRouter."""

    def test_pinning_encapsulation(self):
        """Check the pinning getters and setters."""
        assert not this_thread_is_pinned(), \
            "Thread started out pinned or this_thread_is_pinned() is broken."

        pin_this_thread()
        assert this_thread_is_pinned(), \
            "pin_this_thread() didn't pin the thread."

        unpin_this_thread()
        assert not this_thread_is_pinned(), \
            "Thread remained pinned after unpin_this_thread()."

    def test_pinned_reads(self):
        """Test PinningReplicaRouter.db_for_read() when pinned and when
        not."""
        router = PinningReplicaRouter()

        eq_(router.db_for_read(None), get_replica())

        pin_this_thread()
        eq_(router.db_for_read(None), PRIMARY_DB_ALIAS)

    def test_db_write_decorator(self):
        def read_view(req):
            eq_(router.db_for_read(None), get_replica())
            return HttpResponse()

        @db_write
        def write_view(req):
            eq_(router.db_for_read(None), PRIMARY_DB_ALIAS)
            return HttpResponse()

        router = PinningReplicaRouter()
        eq_(router.db_for_read(None), get_replica())
        write_view(HttpRequest())
        read_view(HttpRequest())


class MiddlewareTests(UnpinningTestCase):
    """Tests for the middleware that supports pinning"""

    def setUp(self):
        super(MiddlewareTests, self).setUp()

        # Every test uses these, so they're okay as attrs.
        self.request = HttpRequest()
        self.middleware = PinningRouterMiddleware()

    def test_pin_on_cookie(self):
        """Thread should pin when the cookie is set."""
        self.request.COOKIES[PINNING_COOKIE] = 'y'
        self.middleware.process_request(self.request)
        assert this_thread_is_pinned()

    def test_unpin_on_no_cookie(self):
        """Thread should unpin when cookie is absent and method is GET."""
        pin_this_thread()
        self.request.method = 'GET'
        self.middleware.process_request(self.request)
        assert not this_thread_is_pinned()

    def test_pin_on_post(self):
        """Thread should pin when method is POST."""
        self.request.method = 'POST'
        self.middleware.process_request(self.request)
        assert this_thread_is_pinned()

    def test_process_response(self):
        """Make sure the cookie gets set on POSTs but not GETs."""

        self.request.method = 'GET'
        response = self.middleware.process_response(
            self.request, HttpResponse())
        assert PINNING_COOKIE not in response.cookies

        self.request.method = 'POST'
        response = self.middleware.process_response(
            self.request, HttpResponse())
        assert PINNING_COOKIE in response.cookies
        eq_(response.cookies[PINNING_COOKIE]['max-age'],
            PINNING_SECONDS)

    def test_attribute(self):
        """The cookie should get set if the _db_write attribute is True."""
        res = HttpResponse()
        res._db_write = True
        response = self.middleware.process_response(self.request, res)
        assert PINNING_COOKIE in response.cookies

    def test_db_write_decorator(self):
        """The @db_write decorator should make any view set the cookie."""
        req = self.request
        req.method = 'GET'

        def view(req):
            return HttpResponse()

        response = self.middleware.process_response(req, view(req))
        assert PINNING_COOKIE not in response.cookies

        @db_write
        def write_view(req):
            return HttpResponse()

        response = self.middleware.process_response(req, write_view(req))
        assert PINNING_COOKIE in response.cookies


class UsePrimaryDBTests(TestCase):
    def test_decorator(self):
        @use_primary_db
        def check():
            assert this_thread_is_pinned()

        unpin_this_thread()
        assert not this_thread_is_pinned()
        check()
        assert not this_thread_is_pinned()

    def test_decorator_resets(self):
        @use_primary_db
        def check():
            assert this_thread_is_pinned()

        pin_this_thread()
        assert this_thread_is_pinned()
        check()
        assert this_thread_is_pinned()

    def test_context_manager(self):
        unpin_this_thread()
        assert not this_thread_is_pinned()
        with use_primary_db:
            assert this_thread_is_pinned()
        assert not this_thread_is_pinned()

    def test_context_manager_resets(self):
        pin_this_thread()
        assert this_thread_is_pinned()
        with use_primary_db:
            assert this_thread_is_pinned()
        assert this_thread_is_pinned()

    def test_context_manager_exception(self):
        unpin_this_thread()
        assert not this_thread_is_pinned()
        with self.assertRaises(ValueError):
            with use_primary_db:
                assert this_thread_is_pinned()
                raise ValueError
        assert not this_thread_is_pinned()

    def test_multithreaded_unpinning(self):
        thread1_lock = Lock()
        thread2_lock = Lock()
        thread1_lock.acquire()
        thread2_lock.acquire()
        orchestrator = Lock()
        orchestrator.acquire()

        pinned = {}

        def thread1_worker():
            with use_primary_db:
                orchestrator.release()
                thread1_lock.acquire()

            pinned[1] = this_thread_is_pinned()

        def thread2_worker():
            pin_this_thread()
            with use_primary_db:
                orchestrator.release()
                thread2_lock.acquire()

            pinned[2] = this_thread_is_pinned()
            orchestrator.release()

        thread1 = Thread(target=thread1_worker)
        thread2 = Thread(target=thread2_worker)

        # thread1 starts, entering `use_primary_db` from an unpinned state
        thread1.start()
        orchestrator.acquire()

        # thread2 starts, entering `use_primary_db` from a pinned state
        thread2.start()
        orchestrator.acquire()

        # thread2 finishes, returning to a pinned state
        thread2_lock.release()
        thread2.join()
        self.assertEqual(pinned[2], True)

        # thread1 finishes, returning to an unpinned state
        thread1_lock.release()
        thread1.join()
        self.assertEqual(pinned[1], False)



from sentry_sdk import configure_scope


class Level:
    FATAL = 'fatal'
    ERROR = 'error'
    WARNING = 'warning'
    INFO = 'info'
    debug = 'debug'


def config_scope(metadata, status, level):
    with configure_scope() as scope:
        scope.set_extra('status', status)
        scope.level = level
        if metadata:
            # set user
            scope.user = metadata.get(
                'user',
                {
                    'username': 'anonymous user',
                }
            )
            # set locale
            locale = metadata.get('locale', 'fa-Ir')
            scope.set_tag("page_locale", locale)



from rest_framework.permissions import BasePermission


class TamarinPermission(BasePermission):

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)



from rest_framework.authentication import BaseAuthentication
from django.contrib.auth.models import AnonymousUser
from datetime import datetime
from jose import jwt
from django.contrib.auth import get_user_model
from django.conf import settings


class TamarinUserTokenAuthentication(BaseAuthentication):
    def authenticate(self, request):
        User = get_user_model()
        user = AnonymousUser()
        try:
            raw_token = str(request.headers['Authorization'])
            if not raw_token.startswith('Bearer'):
                return user, None
            net_token = raw_token.split(' ')
            if not len(net_token) == 2:
                return user, None
            token = net_token[1].strip()
            key = getattr(settings, 'TAMARIN_JWT_KEY', settings.SECRET_KEY)
            token = jwt.decode(token, key)
            user_queryset = User.objects.filter(pk=token['pk'])
            if token['token_type'] == 'access' and user_queryset.exists():
                user = user_queryset.last()
            return user, None
        except Exception as e:
            return user, None


class TamarinTokenAuthentication(BaseAuthentication):
    def authenticate(self, request):
        User = get_user_model()
        user = AnonymousUser()
        try:
            raw_token = str(request.headers['Authorization'])
            if not raw_token.startswith('Bearer'):
                setattr(request, 'crs_client', None)
                return user, None
            net_token = raw_token.split(' ')
            if not len(net_token) == 2:
                setattr(request, 'crs_client', None)
                return user, None
            token = net_token[1].strip()
            key = getattr(settings, 'TAMARIN_JWT_KEY', settings.SECRET_KEY)
            token = jwt.decode(token, key)
            if token['token_type'] == 'access':
                setattr(request, 'crs_client', token)
                user = token
            else:
                setattr(request, 'crs_client', None)
            return user, None
        except Exception as e:
            setattr(request, 'crs_client', None)
            return user, None



import itertools
import random
import warnings
from functools import wraps
import threading

from django.conf import settings

__all__ = ['this_thread_is_pinned', 'pin_this_thread', 'unpin_this_thread',
           'use_primary_db', 'db_write', 'get_replica', 'PRIMARY_DB_ALIAS']

PRIMARY_DB_ALIAS = getattr(settings, 'TAMARIN_PRIMARY_DB', 'default')

replicas = None

_locals = threading.local()


def this_thread_is_pinned():
    """
    Return whether the current thread should send all its reads to the master DB.
    """
    return getattr(_locals, 'pinned', False)


def pin_this_thread():
    """
    Mark this thread as "stuck" to the master for all DB access.
    """
    _locals.pinned = True


def unpin_this_thread():
    """
    Unmark this thread as "stuck" to the master for all DB access.
    If the thread wasn't marked, do nothing.
    """
    _locals.pinned = False


class UsePrimaryDB(object):
    """
    A contextmanager/decorator to use the master database.
    """

    def __call__(self, function):
        @wraps(function)
        def decorator(*args, **kw):
            with self:
                return function(*args, **kw)

        return decorator

    def __enter__(self):
        _locals.old = this_thread_is_pinned()
        pin_this_thread()

    def __exit__(self, exc_type, exc_value, exc_tb):
        if not _locals.old:
            unpin_this_thread()


use_primary_db = UsePrimaryDB()


def mark_as_write(response):
    """
    Mark a response as having done a DB write.
    """
    response._db_write = True
    return response


def db_write(function):
    @wraps(function)
    def _wrapped(*args, **kw):
        with use_primary_db:
            response = function(*args, **kw)
        return mark_as_write(response)

    return _wrapped


def _get_replica_list():
    global replicas
    if replicas is not None:
        return replicas

    dbs = None
    if hasattr(settings, 'REPLICA_DATABASES'):
        dbs = list(settings.REPLICA_DATABASES)

    if not dbs:
        warnings.warn(
            '[tamarin] No replica databases are configured! '
            'You can configure them with the REPLICA_DATABASES setting.',
            UserWarning,
        )
        replicas = itertools.repeat(PRIMARY_DB_ALIAS)
        return replicas

    # Shuffle the list so the first replica isn't slammed during startup.
    random.shuffle(dbs)

    # Set the replicas as test mirrors of the master.
    for db in dbs:
        settings.DATABASES[db].get('TEST', {})['MIRROR'] = PRIMARY_DB_ALIAS

    replicas = itertools.cycle(dbs)
    return replicas


def get_replica():
    """Returns the alias of a replica database."""
    return next(_get_replica_list())



from tamarin.elasticsearch.config import ALLOWED_STATUS, ALLOWED_ALL
from tamarin import status


def validate_status(status_code):
    if ALLOWED_ALL:
        return True
    allowed_condition = [getattr(status, F'is_{method}', None) for method in ALLOWED_STATUS]
    for condition in allowed_condition:
        if condition and condition(status_code):
            return True
    if status_code in ALLOWED_STATUS:
        return True
    return False



import requests


class Client:
    class Method:
        GET = 'get'
        POST = 'post'
        PUT = 'put'
        PATCH = 'patch'
        DELETE = 'delete'

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def set_auth(self, key: str, value: str) -> None:
        """
        Authorization  : {key} {value}
        """
        token = F"{key}{value}"
        setattr(self, 'token', token)

    def set_bearer_token(self, value: str) -> None:
        """
        Authorization  : bearer {value}
        """
        self.set_auth(
            key="Bearer ",
            value=value
        )

    @staticmethod
    def clear_endpoint(old_endpoint: str) -> str:
        new_endpoint = old_endpoint.replace('//', '/')
        new_endpoint = new_endpoint.replace(':/', '://')
        return new_endpoint

    def request(self, method, url, data=None, json=None, query_params=None, **kwargs) -> requests.Response:
        if data is None:
            data = {}
        if json is None:
            json = {}
        if query_params is None:
            query_params = {}
        endpoint = getattr(self, 'url', '') + url
        clear_endpoint = self.clear_endpoint(endpoint)
        headers = kwargs.pop('headers', {})
        headers.update({
            'Authorization': getattr(self, 'token', '')
        })
        if method == Client.Method.GET:
            return self._get(
                url=clear_endpoint,
                query_params=query_params,
                headers=headers,
                **kwargs
            )
        elif method == Client.Method.POST:
            return self._post(
                url=clear_endpoint,
                data=data,
                json=json,
                query_params=query_params,
                headers=headers,
                **kwargs
            )
        elif method == Client.Method.PUT:
            return self._put(
                url=clear_endpoint,
                data=data,
                headers=headers,
                **kwargs
            )
        elif method == Client.Method.PATCH:
            return self._patch(
                url=clear_endpoint,
                data=data,
                headers=headers,
                **kwargs
            )
        elif method == Client.Method.DELETE:
            return self._delete(
                url=clear_endpoint,
                data=data,
                headers=headers,
                **kwargs
            )

    @staticmethod
    def _get(url, query_params, **kwargs) -> requests.Response:
        return requests.get(
            url=url,
            params=query_params,
            **kwargs
        )

    @staticmethod
    def _post(url, data, json, query_params, **kwargs) -> requests.Response:
        return requests.post(
            url=url,
            data=data,
            json=json,
            params=query_params,
            **kwargs
        )

    @staticmethod
    def _put(url, data, **kwargs) -> requests.Response:
        return requests.put(
            url=url,
            data=data,
            **kwargs
        )

    @staticmethod
    def _patch(url, data, **kwargs) -> requests.Response:
        return requests.patch(
            url=url,
            data=data,
            **kwargs
        )

    @staticmethod
    def _delete(url, data, **kwargs) -> requests.Response:
        return requests.delete(
            url=url,
            data=data,
            **kwargs
        )



from Crypto.Cipher import PKCS1_OAEP
from Crypto.PublicKey import RSA
from Crypto.Hash import SHA256
from binascii import hexlify


class RSAEncryption:

    def __init__(self, private_key=None):
        if private_key is None:
            setattr(self, '__private_key__', RSA.generate(2048))
        else:
            setattr(self, '__private_key__', private_key)

    @property
    def private_key(self):
        """
        :return: return private key with 2048 bits
        """
        return getattr(self, '__private_key__').export_key().decode()

    def public_key(self):
        return getattr(self, '__private_key__').publickey().export_key().decode()

    def _import_key(self, public_key):
        return RSA.import_key(public_key)

    def encrypt(self, message, public_key):
        public_key = self._import_key(public_key)
        cipher = PKCS1_OAEP.new(
            key=public_key,
            hashAlgo=SHA256
        )
        encrypted_message = cipher.encrypt(message=message)

        return encrypted_message

    def decrypt(self, encrypted_message):
        private_key = getattr(self, '__private_key__')
        decrypt = PKCS1_OAEP.new(key=private_key)
        decrypted_message = decrypt.decrypt(encrypted_message)
        return decrypted_message



from ...connections.client import Client
from firebase_admin import firestore, initialize_app, credentials
from google.cloud.firestore import SERVER_TIMESTAMP, ArrayUnion, ArrayRemove, Increment, DELETE_FIELD
from uuid import uuid4
import os
from django.conf import settings

_DEFAULT_APP_NAME = 'FIRESTORE_DEFAULT'


class FirestoreClient(Client):
    __client__ = None

    __firestore_client__ = None

    __credential__ = credentials.Certificate(
        os.environ.get(
            'GOOGLE_APPLICATION_CREDENTIALS',
            getattr(
                settings,
                'GOOGLE_APPLICATION_CREDENTIALS'
            )
        )
    )

    @staticmethod
    def get_instance():
        return FirestoreClient.__firestore_client__ or FirestoreClient()

    def __init__(self):
        if FirestoreClient.__firestore_client__:
            raise Exception("Firestore is singleton, just one instance")
        else:
            credential = FirestoreClient.__credential__
            options = getattr(settings, 'FIREBASE_APP_OPTIONS', None)
            name = getattr(settings, 'FIREBASE_APP_NAME', _DEFAULT_APP_NAME)
            app = initialize_app(credential=credential, options=options, name=name)
            FirestoreClient.__client__ = firestore.client(app=app)
            FirestoreClient.__firestore_client__ = self

    def request(self, method, url, data=None, json=None, query_params=None, **kwargs):
        raise PermissionError("Firestore client request method not allowed")

    @property
    def client(self):
        return self.__client__

    def send(self, collection, data, document_id=None):
        client = self.client
        collection = client.collection(collection)
        if document_id is None:
            document_id = str(uuid4())
        document = collection.document(document_id)
        document.set(data)

    def update(self, collection, document_id, data):
        client = self.client
        collection = client.collection(collection)
        document = collection.document(document_id)
        document.update(data)

    def list(self, collection):
        client = self.client
        collection = client.collection(collection)
        documents = collection.stream()
        response = [{document.id: document.to_dict()} for document in documents]
        return response

    def delete(self, collection, document_id):
        client = self.client
        collection = client.collection(collection)
        document = collection.document(document_id)
        document.delete()

    @property
    def timestamp(self):
        return SERVER_TIMESTAMP

    @staticmethod
    def add_or_update_array_item(array_field, item):
        return {
            array_field: ArrayUnion([item])
        }

    @staticmethod
    def remove_array_item(array_field, item):
        return {
            array_field: ArrayRemove([item])
        }

    @staticmethod
    def increment_numeric_value(field, increment):
        return {
            field: Increment(increment)
        }

    @staticmethod
    def delete_field(field):
        return {
            field: DELETE_FIELD
        }



from django.db import models
from django.contrib.auth import get_user_model


class Log(models.Model):
    class Category(models.TextChoices):
        RESERVE = 'reserve'

    user_id = models.CharField(
        max_length=1000
    )

    @property
    def user(self):
        try:
            return get_user_model().objects.get(pk=self.user_id)
        except:
            return None

    category = models.CharField(max_length=1000, choices=Category.choices)
    description = models.TextField()
    reference_url = models.URLField(null=True, blank=True)
    created_at = models.DateTimeField(auto_created=True, auto_now_add=True)

    def __str__(self):
        return F"{self.category}"

    class Meta:
        ordering = ('-created_at',)
        verbose_name = 'log'
        verbose_name_plural = 'log'


class Report(models.Model):
    index = models.CharField(max_length=1000)
    status = models.PositiveSmallIntegerField()
    data = models.JSONField(null=True, blank=True)
    text = models.TextField(null=True, blank=True)
    created_at = models.DateTimeField(auto_created=True, auto_now_add=True)

    def __str__(self):
        return F"{self.index} - {self.created_at}"

    class Meta:
        ordering = ('-created_at', 'index')



import setuptools

with open("tamarin/README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="tamarin-utils",
    version="0.10.0",
    author="Lamasoo team",
    author_email="tech@lamasoo.com",
    description="Lamasoo utils for developing web application",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://git.lamasoo.com/crs-agency/tamarin",
    install_requires=[
        'requests',
        'djangorestframework',
        'elasticsearch[async]',
        'celery',
        'sentry-sdk',
        'python-dateutil',
        'firebase-admin',
        'google-api-python-client',
        'python-jose',
        'djangorestframework-simplejwt',
        'pycryptodome',
        'django-nose',
        'python-dateutil',
        'python-telegram-bot',
        'Pyrogram',
        'TgCrypto',
    ],
    packages=setuptools.find_packages(),
    classifiers=[
        'Environment :: Web Environment',
        "Programming Language :: Python :: 3",
        'License :: OSI Approved :: MIT License',
        "Operating System :: OS Independent",
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
    ],
    python_requires='>=3.6',
)

# py setup.py sdist bdist_wheel
# twine upload dist/*



from .utils import get_replica, this_thread_is_pinned, PRIMARY_DB_ALIAS
from .replica_router import ReplicaRouter
from django.conf import settings


class PinningReplicaRouter(ReplicaRouter):

    def db_for_read(self, model, **hints):
        return PRIMARY_DB_ALIAS if this_thread_is_pinned() else get_replica()



from .utils import get_replica, PRIMARY_DB_ALIAS


class ReplicaRouter(object):
    """Router that sends all reads to a replica, all writes to default."""

    def db_for_read(self, model, **hints):
        """Send reads to replicas in round-robin."""
        return get_replica()

    def db_for_write(self, model, **hints):
        """Send all writes to the master."""
        return PRIMARY_DB_ALIAS

    def allow_relation(self, obj1, obj2, **hints):
        """Allow all relations, so FK validation stays quiet."""
        return True

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        return db == PRIMARY_DB_ALIAS

    def allow_syncdb(self, db, model):
        """Only allow syncdb on the master."""
        return db == PRIMARY_DB_ALIAS



from .authentication_backend import TamarinTokenAuthentication, TamarinUserTokenAuthentication
from .is_authenticated import IsAuthenticated, IsUserAuthenticated
from .is_staff import IsStaff, IsUserStaff
from .has_permission import HasPermission, HasUserPermission
from .is_member import IsMember, IsUserMember
from .token import TamarinTokenSerializer, TamarinTokenObtainPairView, TamarinRefreshView
from rest_framework_simplejwt.tokens import AccessToken



from rest_framework.permissions import BasePermission
from django.contrib.auth import get_user_model
from django.db.models import Q


class IsMember(BasePermission):

    def has_permission(self, request, view):
        groups = getattr(self, 'filter')
        if groups:
            if type(groups) == list:
                user_groups = request.user.get('groups')
                for group in groups:
                    if not (group in user_groups):
                        return False
                return True
            elif type(groups) == str:
                return groups in request.user.get('groups')
            else:
                raise AttributeError('groups type is not list or string')
        else:
            raise AttributeError('groups does not set')


class IsUserMember(BasePermission):

    def has_permission(self, request, view):
        User = get_user_model()
        groups = getattr(self, 'filter')
        if groups:
            if type(groups) == list:
                queryset = User.objects.filter(
                    Q(pk=request.user.pk),
                    Q(groups__name__in=groups)
                )
                return queryset.exists()
            elif type(groups) == str:
                queryset = User.objects.filter(
                    Q(pk=request.user.pk),
                    Q(groups__name=groups)
                )
                return queryset.exists()
            else:
                raise AttributeError('groups type is not list or string')
        else:
            raise AttributeError('groups does not set')



# Generated by Django 3.0.2 on 2020-10-06 11:55

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_created=True, auto_now_add=True)),
                ('user_id', models.CharField(max_length=1000)),
                ('category', models.CharField(choices=[('reserve', 'Reserve')], max_length=1000)),
                ('description', models.TextField()),
                ('reference_url', models.URLField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'log',
                'verbose_name_plural': 'log',
                'ordering': ('-created_at',),
            },
        ),
    ]



from functools import wraps
from typing import Dict


def mock(func_list: dict, condition: str):
    """
    :return: return response and say that request has error or not
    """

    def decorator(func):
        @wraps(func)
        def inner(*args, **kwargs):
            if condition:
                if condition.lower() == 'test':
                    return func_list.get('test', func)(*args, **kwargs)
                elif condition.lower() == 'development':  # pragma: no cover
                    return func_list.get('development', func)(*args, **kwargs)
                elif condition.lower() == 'staging':  # pragma: no cover
                    return func_list.get('staging', func)(*args, **kwargs)
                elif condition.lower() == 'production':  # pragma: no cover
                    return func_list.get('production', func)(*args, **kwargs)
            return func(*args, **kwargs)

        return inner

    return decorator
